"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultPollingListenerBatchSizeStrategy = void 0;
/**
 * The default batch size strategy returns the configured value from the
 * `nextMessagesBatchSize`. Default is 5. But the first few times until the
 * batch size is reached it will tell to return only one message. This protects
 * against poisonous messages: if 5 messages would be taken during startup all
 * those 5 would be marked as poisonous if one of them fails.
 */
const defaultPollingListenerBatchSizeStrategy = (config) => {
    var _a;
    const max = (_a = config.settings.nextMessagesBatchSize) !== null && _a !== void 0 ? _a : 5;
    let callsSinceStart = 1;
    return () => {
        let batchSize = max;
        if (callsSinceStart <= max) {
            batchSize = 1;
            callsSinceStart++;
        }
        return Promise.resolve(batchSize);
    };
};
exports.defaultPollingListenerBatchSizeStrategy = defaultPollingListenerBatchSizeStrategy;
//# sourceMappingURL=batch-size-strategy.js.map